/*
 * DefaultTest.cs
 * Copyright (C) 2002 Mike Krueger, icsharpcode
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer. 
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution. 
 * 
 * - Neither the name of icsharpcode nor the names of its contributors may 
 *   be used to endorse or promote products derived from this software without specific 
 *   prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
using System;
using System.Reflection;
using System.Collections;

namespace ICSharpCode.SharpUnit.Environment {
	
	/// <summary>
	/// Default implementation of the <see cref="ITest"/> interface.
	/// </summary>
	public class DefaultTest : MarshalByRefObject, ITest
	{
		ITestSuite suite;
		MethodInfo methodInfo;
		string     description;
		
		public string Name {
			get {
				return methodInfo.Name;
			}
		}
		
		public string Description {
			get {
				return description;
			}
		}
		
		public DefaultTest(ITestSuite suite, MethodInfo methodInfo)
		{
			this.suite      = suite;
			this.methodInfo = methodInfo;
			this.description = ((TestMethodAttribute)Attribute.GetCustomAttribute(methodInfo, typeof(TestMethodAttribute))).Description;
		}
		
		public void Run()
		{
			try {
 				suite.Type.InvokeMember(methodInfo.Name, BindingFlags.InvokeMethod, null, suite.TestClass, new object [] {});
			} catch (Exception e) {
				bool validException = false;
				ExceptExceptionAttribute[] exceptedExceptions = (ExceptExceptionAttribute[])Attribute.GetCustomAttributes(methodInfo, typeof(ExceptExceptionAttribute));
				foreach (ExceptExceptionAttribute eea in exceptedExceptions) {
					if (e.InnerException != null && e.InnerException.GetType().Equals(eea.ExceptionType)) {
						validException = true;
					}
				}
				
				if (!validException) {
					throw e;
				}
			}
		}
	}
}
